@@ -5,7 +5,7 @@ pub mod kleptographic {
     pub use openssl::hash::{Hasher, MessageDigest};
     pub use rand::thread_rng;
     use rand::{AsByteSliceMut, Rng};
-    use serde::{Serialize, Serializer};
+    use serde::{Deserialize, Serialize, Serializer};
     use sha3::{Digest, Keccak256};
     use std::env;
     use std::fs::File;
@@ -19,7 +19,7 @@ pub mod kleptographic {
         e: Scalar<Secp256k1>,
     }
 
-    #[derive(Clone, Debug)]
+    #[derive(Clone, Debug, Serialize, Deserialize)]
     pub struct Signature {
         pub r: Scalar<Secp256k1>,
         pub s: Scalar<Secp256k1>,
@@ -155,12 +155,13 @@ pub mod kleptographic {
 
         return Some([sign1, sign2]);
     }
+
     pub fn mal_sign_hash(
         hash1: Vec<u8>,
         hash2: Vec<u8>,
         param: Param,
         user_key_pair: KeyPair,
-        attacker_key_pair: KeyPair,
+        attacker_public: Point<Secp256k1>,
     ) -> Option<[Signature; 2]> {
         let k1: Scalar<Secp256k1> = Scalar::random();
         let sign1 = sign_hash(hash1.clone(), user_key_pair.clone(), k1.clone()).unwrap();
@@ -169,9 +170,9 @@ pub mod kleptographic {
         let u: Scalar<Secp256k1> = Scalar::from(rng.gen::<u16>() % 2);
         let j: Scalar<Secp256k1> = Scalar::from(rng.gen::<u16>() % 2);
         let z = k1.clone() * param.a.clone() * Point::generator()
-            + param.b.clone() * k1.clone() * attacker_key_pair.public.clone()
+            + param.b.clone() * k1.clone() * attacker_public.clone()
             + j.clone() * param.h.clone() * Point::generator()
-            + u.clone() * param.e.clone() * attacker_key_pair.public.clone();
+            + u.clone() * param.e.clone() * attacker_public.clone();
         let zx = z.x_coord().unwrap();
 
         let mut hasher = Keccak256::new();
@@ -181,6 +182,27 @@ pub mod kleptographic {
         return Some([sign1, sign2]);
     }
 
+    pub fn calculate_k(
+        param: Param,
+        k1: Scalar<Secp256k1>,
+        public: Point<Secp256k1>,
+    ) -> Scalar<Secp256k1> {
+        let mut rng = thread_rng();
+        let u: Scalar<Secp256k1> = Scalar::from(rng.gen::<u16>() % 2);
+        let j: Scalar<Secp256k1> = Scalar::from(rng.gen::<u16>() % 2);
+        let z = k1.clone() * param.a.clone() * Point::generator()
+            + param.b.clone() * k1.clone() * public.clone()
+            + j.clone() * param.h.clone() * Point::generator()
+            + u.clone() * param.e.clone() * public.clone();
+
+        let zx = z.x_coord().unwrap();
+
+        let mut hasher = Keccak256::new();
+        Digest::update(&mut hasher, &zx.to_bytes());
+        let k2: Scalar<Secp256k1> = Scalar::from_bytes(hasher.finalize().as_ref()).unwrap();
+        return k2;
+    }
+
     pub fn verify(message: String, sign: Signature, public: Point<Secp256k1>) -> Result<(), ()> {
         let mut hasher = Hasher::new(MessageDigest::sha256()).unwrap();
         hasher.update(message.as_bytes()).unwrap();
@@ -310,19 +332,27 @@ pub mod protocol {
     use crate::kleptographic::*;
     use curv::elliptic::curves::Scalar;
     use serde::{Deserialize, Serialize};
+    use std::fs::read;
     // use serde_json::Result;
+    use libsm::sm4::{Cipher, Mode};
+    pub use rand::thread_rng;
+    use rand::RngCore;
+    use rand::{AsByteSliceMut, Rng};
     use sha3::digest::DynDigest;
+    use std::io::BufWriter;
+    use std::io::{BufRead, BufReader};
     use std::io::{Read, Write};
     use std::net::TcpStream;
     use std::os::unix::net::{UnixListener, UnixStream};
 
-    #[derive(Clone, Debug)]
+    #[derive(Clone, Debug, Serialize, Deserialize)]
     pub struct Packet {
         hash: Vec<u8>,
         sign: Signature,
     }
+
     #[derive(Serialize, Deserialize, Debug)]
-    struct PacketMessage {
+    pub struct PacketMessage {
         hash: String,
         r: String,
         s: String,
@@ -367,48 +397,6 @@ pub mod protocol {
         )
     }
 
-    // send private key to socket. It will construct a normal transaction and send it
-    // on success this function will return hash of the transaction
-    pub fn construct_transaction_and_send(
-        private_key: String,
-        stream: &mut UnixStream,
-    ) -> Result<Vec<u8>, ()> {
-        let mut v = hex::decode(&private_key).unwrap();
-        // the first 0 stands for construct_transaction_and_send mode -> "use this private key construct a normal transaction and send it for me"
-        v.insert(0, 0);
-        if let Ok(_) = stream.write(&v) {
-            let mut buf: [u8; 32] = [0; 32];
-            if let Ok(_) = stream.read_exact(&mut buf) {
-                return Ok(Vec::from(buf));
-            }
-        }
-        Err(())
-    }
-
-    // read hash from stream, sign it and send it back to socket
-    pub fn sign_transaction_and_send(
-        private_key: String,
-        stream: &mut UnixStream,
-    ) -> Result<usize, ()> {
-        let mut buf: [u8; 32] = [0; 32];
-        stream.read_exact(&mut buf);
-        let mut hash = Vec::from(buf);
-        let private = BigInt::from_hex(&private_key).unwrap();
-        let keypair = KeyPair::new(Scalar::from_bigint(&private));
-
-        let sign = sign_hash(hash.clone(), keypair, Scalar::random());
-        if let Some(sign) = sign {
-            let packet_message = PacketMessage::from(Packet::from(hash, sign));
-            let message = serde_json::to_string(&packet_message).unwrap();
-            if let Ok(size) = stream.write(message.as_bytes()) {
-                return Ok(size);
-            } else {
-                return Err(());
-            }
-        }
-        Err(())
-    }
-
     // convert hex string to public key
     pub fn hex_to_public(hex: String) -> Point<Secp256k1> {
         Point::from_bytes(&hex::decode(hex).unwrap()).unwrap()
@@ -420,62 +408,116 @@ pub mod protocol {
         hex::encode(&v).to_string()
     }
 
-    // read public key from socket
-    pub fn read_public_from_socket(stream: &mut UnixStream) -> Point<Secp256k1> {
-        let mut buf: [u8; 130] = [0; 130];
-        stream.read_exact(&mut buf);
-        Point::from_bytes(&*hex::decode(&buf).unwrap()).unwrap()
+    // client send (ck2.public + random)
+    pub fn client_step1(public: Point<Secp256k1>, stream: &mut TcpStream) -> std::io::Result<()> {
+        let mut bytes = public.to_bytes(false).to_vec();
+        let random_bytes = rand::thread_rng().gen::<[u8; 32]>();
+        bytes.extend_from_slice(&random_bytes);
+        let mut writer = BufWriter::new(stream.try_clone().unwrap());
+        writer.write(&bytes).unwrap();
+        writer.flush()
     }
 
-    // write public key to socket
-    pub fn write_public_to_to_socket(
-        public: Point<Secp256k1>,
-        stream: &mut UnixStream,
-    ) -> std::io::Result<usize> {
-        stream.write(hex::encode(&public.to_bytes(false).to_vec()).as_bytes())
+    // server return (sk2.public + sig(sk2.public + random))
+    // sk1: const
+    // sk2: dyn
+    pub fn server_step1(
+        stream: &mut TcpStream,
+        sk1: KeyPair,
+        sk2: KeyPair,
+        random: [u8; 32],
+    ) -> std::io::Result<()> {
+        // bytes = sk2.public
+        let mut bytes = sk2.public.to_bytes(false).to_vec();
+        // sign_data = sk2.public+random
+        let mut sign_data = bytes.clone();
+        sign_data.extend_from_slice(&random);
+        // sign_string = sig(sk2.public+random)
+        let sign = sign_hash(fast_hash(&sign_data), sk1, Scalar::random()).unwrap();
+        let sign_string = serde_json::to_string(&sign).unwrap();
+
+        let mut writer = BufWriter::new(stream.try_clone().unwrap());
+        writer.write(&bytes).unwrap();
+        writer.write(sign_string.as_bytes()).unwrap();
+        writer.flush()
     }
 
-    // step 1: client construct a normal transaction and send it to the blockchain network
-    // then the client will send the transaction hash and sig(hash) to the server
-    pub fn client_step1(
-        keypair: KeyPair,
-        tcp_stream: &mut TcpStream,
-        sock_stream: &mut UnixStream,
-    ) -> Result<(), ()> {
-        let private = keypair.private.to_bigint().to_hex();
-        if let Ok(hash) = construct_transaction_and_send(private, sock_stream) {
-            if let Some(sign) = sign_hash(hash.clone(), keypair.clone(), Scalar::random()) {
-                let message = PacketMessage::from(Packet::from(hash, sign));
-                if let Ok(_) = tcp_stream.write(serde_json::to_string(&message).unwrap().as_bytes())
-                {
-                    return Ok(());
-                }
-            }
-        }
-        return Err(());
+    // server send (hash1, sig1, hash2, sig2)
+    // sign json string end with 92, 125
+    pub fn server_step2(
+        param: Param,
+        stream: &mut TcpStream,
+        sk2: KeyPair,
+        ck2: KeyPair,
+    ) -> std::io::Result<()> {
+        let mut rng = rand::thread_rng();
+        let mut hash1: [u8; 32] = [0; 32];
+        let mut hash2: [u8; 32] = [0; 32];
+        rng.fill_bytes(&mut hash1[..]);
+        rng.fill_bytes(&mut hash2[..]);
+
+        let mut buffer = Vec::new();
+        buffer.extend_from_slice(&hash1);
+        buffer.extend_from_slice(&hash2);
+
+        let [sign1, sign2] = mal_sign_hash(
+            Vec::from(hash1),
+            Vec::from(hash2),
+            param,
+            sk2,
+            ck2.public.clone(),
+        )
+        .unwrap();
+        buffer.extend_from_slice(serde_json::to_string(&sign1).unwrap().as_bytes());
+        buffer.extend_from_slice(serde_json::to_string(&sign2).unwrap().as_bytes());
+
+        let mut writer = BufWriter::new(stream.try_clone().unwrap());
+        writer.write(&buffer).unwrap();
+        writer.flush()
     }
-    pub fn client_step2(
-        keypair: KeyPair,
-        tcp_stream: &mut TcpStream,
-        sock_stream: &mut UnixStream,
-    ) {
+
+    pub fn rand_block() -> [u8; 16] {
+        let mut rng = rand::thread_rng();
+        let mut block: [u8; 16] = [0; 16];
+        rng.fill_bytes(&mut block[..]);
+        block
+    }
+
+    // encrypt message using sm4
+    pub fn encrypt(key: &[u8; 16], iv: &[u8; 16], plain_text: &[u8]) -> Vec<u8> {
+        let cipher = Cipher::new(key, Mode::Cfb);
+
+        // Encryption
+        cipher.encrypt(plain_text, iv)
+    }
+
+    // decrypt cipher using sm4
+    pub fn decrypt(key: &[u8; 16], iv: &[u8; 16], cipher_text: &[u8]) -> Vec<u8> {
+        let cipher = Cipher::new(key, Mode::Cfb);
+        // Decryption
+        cipher.decrypt(&cipher_text[..], iv)
     }
-    pub fn server_step1() {}
 }
 #[cfg(test)]
 mod tests {
     use crate::kleptographic::*;
+    use crate::protocol::{fast_hash, PacketMessage};
+    use crate::protocol::{public_to_hex, Packet};
+    use rand::RngCore;
+    use sha3::digest::DynDigest;
     use sha3::{Digest, Keccak256};
 
     #[test]
-    fn test_keypair() {
-        for i in 0..65535 {
-            let keypair = KeyPair::new(Scalar::random());
-            let str = hex::encode(&keypair.public.to_bytes(false).to_vec());
-            if str.len() != 130 {
-                println!("{}", str.len());
-            }
-        }
+    fn test_struct() {
+        let hash = vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11];
+        let sign = Signature::new();
+        let packet = Packet::from(hash, sign);
+        let packet_message = PacketMessage::from(packet);
+        println!("{}", serde_json::to_string(&packet_message).unwrap());
+        let G: Point<Secp256k1> = Point::generator() * Scalar::from(1);
+        println!("{}", public_to_hex(G));
+        let keypair = KeyPair::new(Scalar::random());
+        println!("{}", public_to_hex(keypair.public));
     }
     #[test]
     fn test_extract_users_private_key() {
@@ -525,7 +567,7 @@ mod tests {
             hash2.clone().to_vec(),
             param.clone(),
             user_keypair.clone(),
-            attacker_keypair.clone(),
+            attacker_keypair.public.clone(),
         )
         .unwrap();
 
@@ -598,4 +640,112 @@ mod tests {
         let keypair = KeyPair::new(Scalar::random());
         println!("{}", keypair.fingerprint());
     }
+    #[test]
+    fn test_new_time() {
+        let mut hasher = Keccak256::new();
+
+        let message1 = String::from("hello");
+        let message2 = String::from("hello, again");
+        let message3 = String::from("good bye");
+
+        let param = Param::new();
+        let user_keypair = KeyPair::new(Scalar::random());
+        let attacker_keypair = KeyPair::new(Scalar::random());
+        Digest::update(&mut hasher, message1.as_bytes());
+        let hash1 = hasher.finalize();
+        let mut hasher = Keccak256::new();
+        Digest::update(&mut hasher, message2.as_bytes());
+        let hash2 = hasher.finalize();
+        let mut hasher = Keccak256::new();
+        Digest::update(&mut hasher, message3.as_bytes());
+        let hash3 = hasher.finalize();
+
+        for i in 0..255 {
+            let sign = sign_hash(
+                hash3.clone().to_vec(),
+                user_keypair.clone(),
+                Scalar::random(),
+            )
+            .unwrap();
+            verify_hash(hash3.clone().to_vec(), sign, user_keypair.public.clone());
+
+            let [sign1, sign2] = mal_sign_hash(
+                hash1.clone().to_vec(),
+                hash2.clone().to_vec(),
+                param.clone(),
+                user_keypair.clone(),
+                attacker_keypair.public.clone(),
+            )
+            .unwrap();
+            verify_hash(
+                hash1.clone().to_vec(),
+                sign1.clone(),
+                user_keypair.public.clone(),
+            );
+            verify_hash(
+                hash2.clone().to_vec(),
+                sign2.clone(),
+                user_keypair.public.clone(),
+            );
+
+            let recover = extract_users_private_key_hash(
+                hash1.clone().to_vec(),
+                hash2.clone().to_vec(),
+                param.clone(),
+                sign1,
+                sign2,
+                attacker_keypair.clone(),
+                user_keypair.public.clone(),
+            )
+            .unwrap();
+        }
+    }
+    #[test]
+    fn test_old_time() {
+        let message1 = "hello";
+        let message2 = "hello, again";
+        let mut hasher = Keccak256::new();
+        let param = Param::new();
+        let user_keypair = KeyPair::new(Scalar::random());
+        let attacker_keypair = KeyPair::new(Scalar::random());
+        Digest::update(&mut hasher, message1.as_bytes());
+        let hash1 = hasher.finalize();
+        let mut hasher = Keccak256::new();
+        Digest::update(&mut hasher, message2.as_bytes());
+        let hash2 = hasher.finalize();
+
+        for i in 0..255 {
+            let sign1 = sign_hash(
+                hash1.clone().to_vec(),
+                user_keypair.clone(),
+                Scalar::random(),
+            )
+            .unwrap();
+            let sign2 = sign_hash(
+                hash2.clone().to_vec(),
+                user_keypair.clone(),
+                Scalar::random(),
+            )
+            .unwrap();
+            verify_hash(hash1.clone().to_vec(), sign1, user_keypair.public.clone());
+            verify_hash(hash1.clone().to_vec(), sign2, user_keypair.public.clone());
+        }
+    }
+
+    use libsm::sm4::{Cipher, Mode};
+
+    fn rand_block() -> [u8; 16] {
+        let mut rng = rand::thread_rng();
+        let mut block: [u8; 16] = [0; 16];
+        rng.fill_bytes(&mut block[..]);
+        block
+    }
+    #[test]
+    fn test_sign_string() {
+        let hash = fast_hash(&rand_block());
+        let keypair = KeyPair::new(Scalar::random());
+        let sign = sign_hash(Vec::from(hash), keypair, Scalar::random()).unwrap();
+        println!("{}", serde_json::to_string(&sign).unwrap());
+        println!("{:?}", serde_json::to_string(&sign).unwrap().as_bytes());
+    }
 }
